/**********************************************************************************************/

/* function to get path lengths from ray paths  */

/**********************************************************************************************/

#include <math.h>
#include <stdio.h>
#include "invsubs.h"

void GetLengths(float *xp, float *zp, int nsegs, int nx, int nz, float x_inc, float z_inc, float *lengths)
{
	int             i, j, lastcell, cell;
	float           arc_length = 0, dx, dz;
	int             ixb, izb;


	for (i = 0; i < nx * nz; i++)
		lengths[i] = 0;

	/* find which cell you are in */
	ixb = (int) (xp[0] / x_inc);
	izb = (int) (zp[0] / z_inc);
	lastcell = izb * nx + ixb;
	arc_length = 0;

	for (i = 1; i < nsegs; i++) {
		dz = (zp[i] - zp[i - 1]);
		dx = (xp[i] - xp[i - 1]);
		arc_length += sqrt(dz * dz + dx * dx);
		cell = ((int) (zp[i] / z_inc)) * nx + ((int) (xp[i] / x_inc));

		if (cell != lastcell) {
			lastcell = cell;
			lengths[lastcell] = arc_length;
			arc_length = 0;
		}
	}
	/* finish up in the last cell */
	if (arc_length != 0)
		lengths[cell] = arc_length;

}

/**********************************************************************************************/

/* function to get great-circle arc lengths in each cell  */

/**********************************************************************************************/

void GetGCARCLengths(float *xp, float *zp, float *lats, float *lons, int nsegs, int nx, int nz, float x_inc, float z_inc, float *lengths)
{
	int             i, j, lastcell, cell;
	float           arc_length = 0, dx, dz, az,total_length;
	int             ixb, izb;
	int             i_enter;


	for (i = 0; i < nx * nz; i++)
		lengths[i] = 0;
	total_length = 0;

	/* find which cell you are in */
	ixb = (int) (xp[0] / x_inc);
	izb = (int) (zp[0] / z_inc);
	lastcell = izb * nx + ixb;
	arc_length = 0;
	i_enter = 0;
	

	for (i = 1; i < nsegs; i++) {
		dz = (zp[i] - zp[i - 1]);
		dx = (xp[i] - xp[i - 1]);
		cell = ((int) (zp[i] / z_inc)) * nx + ((int) (xp[i] / x_inc));

		if (cell != lastcell) {
			delta_az(&lats[i_enter], &lons[i_enter], &lats[i], &lons[i], &arc_length, &az);
			lengths[lastcell] = arc_length * 111.19;
			i_enter = i;
			total_length += arc_length * 111.19;
			lastcell = cell;
		}
	}
	/* finish up in the last cell */
	if (i_enter != nsegs - 1) {
		delta_az(&lats[i_enter], &lons[i_enter], &lats[nsegs - 1], &lons[nsegs - 1], &arc_length, &az);
		lengths[cell] = arc_length * 111.19;
		total_length += arc_length * 111.19;
	}
}
